
#include <string.h>
#include <stdlib.h>
#include <stdio.h>
#include <ctype.h>

/*=============================================================================
 * WORD
 */
#define MAXWORD	32
class wordC { public: char S[MAXWORD]; };
static wordC Words[256*1024];
static wordC* WordsEnd = Words;



/*=============================================================================
 * PHONEM
 *
 *    per ogni N
 *     per ogni fonema N che trovi nelle parole
 *	se non l'hai gi in lista
 *	    aggiungi
 *	altrimenti
 *	    incrementa contatore
 */
class phonemC { public:
    phonemC*	Next;
    char	Sym[7];
    char	Size;
    long	Cnt;

    long*	Cross;

    phonemC(char* sym, int size)
    {
	Next = 0;
	Cnt = 1;

	int i;
	for(i = 0; i < size; i++) Sym[i] = sym[i];
	Sym[i] = '\0';
	Size = size;
    }
};





/*=============================================================================
 * PHONEM LIST
 */
class phonemListC { public:
    phonemC*  Hook;
    phonemC** Last;


    phonemListC()
    {
	Hook = 0;
	Last = &Hook;
    }


    void Add(char* sym, int size)
    {
	// is phonem already existing?
	phonemC* p;
	for(p = Hook; p && strncmp(p->Sym, sym, size); p = p->Next);

	if(p)		// yes, increase count
	    p->Cnt++;
	else		// no, add to list
	{
	    *Last = new phonemC(sym, size);
	    Last = &(*Last)->Next;
	}
    }


} phonemListA[4]; // 2,3,4,5 -phonems





/*=============================================================================
 * ADD
 */
void addWordPhonems(char* word)
{
 int l = strlen(word);

 for(int s = 0; s < l; s++)		// for all chars in the word
 {
    int maxn = l - s;
    if(maxn > 5) maxn = 5;


    // ignore all 2-phonems at word beginning
    int q = 0;
    if(s == 0) q = 1;


    for(int n = 2 + q; n <= maxn; n++)	// for all n phonem sizes
	phonemListA[n -2].Add(word + s, n);
 }
}





/*=============================================================================
 * REMOVE COUNTS
 */
void removeCountsFrom(phonemC* targ, phonemListC* l)
{
 int s = targ->Size;

 for(phonemC* p = l->Hook; p; p = p->Next)
    for(char* c = p->Sym; *c; c++)
	if(!strncmp(targ->Sym, c, s))
	    targ->Cnt -= p->Cnt;
}





/*=============================================================================
 * CROSS REFERENCE
    // string starts from word beginning (from main() function)
	//-- recurse
	//for any phonem matching string
	    // if parent phonem
		// array [parent phonem] [phonem] ++
	    // string starts at phonem end -> recurse
 */
//static int** crossTable = 0;
//static int crossTSize = 0;


void phonemCrossReference(char* s, phonemC* parent)
{
 for(phonemC* p = phonemListA->Hook; p; p = p->Next)
    if(!strncmp(s, p->Sym, p->Size))
    {
	if(parent) parent->Cross[p->Cnt]++;
	if(s[p->Size])
	    phonemCrossReference(s + p->Size, p);
    }
}






/*=============================================================================
 * MAIN
 */
int main(int argc, char** argv)
{
 FILE* f = fopen(argv[1], "rt");
 if(!f) return 0;


 // load words and phonems
 char bf[100];
 while(fgets(bf, sizeof(bf), f))
 {
    for(char* c = bf; *c; c++)
	*c = tolower(*c);

    if(char* c = strchr(bf, '\n')) *c = '#';

    WordsEnd->S[0] = '@';
    strncpy(WordsEnd->S+1, bf, MAXWORD);
    addWordPhonems(WordsEnd->S);
    WordsEnd++;
 }



 // to eliminate redundancy between phonems of different grade,
 // before deciding which phonems to eliminate, all phonems' counts
 // are decreased by the count of higher grade phonems containing the smaller
 // phonem.
 //
 // after that, phonems with a low count will be eliminated

 // for any list, (from higher to lower phonem grade)
    // for any phonem
	// strip counts of higher grade phonems containing the phonem itself
    // eliminate phonems with insufficent counts

 for(int n = 5; n >= 2; n--)
 {
    phonemListC* l = phonemListA + n -2;


    // remove counts from higher grade phonems
    for(phonemC* p = l->Hook; p; p = p->Next)
	for(int g = 5; g > n; g--)
	    removeCountsFrom(p, phonemListA + g -2);


    // count total occourrences
    long sum = 0;
    long cnt = 0;

    for(phonemC* p = l->Hook; p; p = p->Next)
    {
	sum += p->Cnt;
	cnt++;
    }


    // define deletion threshold
    int w = n-1;
//    int threshold = w * w * sum / cnt;
//@@ C'e bisogno di una statistica piu accurata per determinare il threshold
    int threshold = 11 * sum / cnt;



    //
    long passcnt = 0;
    for(phonemC* p = l->Hook; p; p = p->Next)
	if(p->Cnt < threshold)
	    p->Size = 0;	// mark for deletion
	else
	    passcnt++;


    // remove marked phonems
    phonemC** h = &l->Hook;
    while(phonemC* p = *h)
	if(p->Size) h = &p->Next;
	else
	{
	    *h = p->Next;	// remove p from chain
	    delete p;		// free memory
	}


    printf("%d-phonem: sum %6d, occ %5d, avg = %4d (%d pass)\n",
	 n, sum, cnt, threshold, passcnt);
 }




 // chain the phonem lists under phonemListA
 // so that phonemListA->Hook hooks for al phonems
 for(int i = 1; i < 4; i++)
    *phonemListA[i-1].Last = phonemListA[i].Hook;




 // phonem counts will now be used as ID
 int cnt = 0;
 for(phonemC* p = phonemListA->Hook; p; p = p->Next)
    p->Cnt = cnt++;

 printf("%d selected phonems\n", cnt);

 // create and zero table
 long crossTable[cnt*cnt];
 for(int x = 0; x < cnt*cnt; x++) crossTable[x] = 0;
 for(phonemC* p = phonemListA->Hook; p; p = p->Next)
    p->Cross = crossTable + p->Cnt*cnt;

 // build cross phonem table
 for(wordC* w = Words; w < WordsEnd; w++)
    phonemCrossReference(w->S, 0);



 // remove columns with low counts
 long sum = 0;
 for(int x = 0; x < cnt*cnt; x++) sum += crossTable[x];

 long threshold = 3*sum/cnt;



 // is there enough parents value for this phonem?
 for(phonemC* p = phonemListA->Hook; p; p = p->Next) if(*p->Sym != '@')
 {
    long s = 0;
    for(phonemC* pa = phonemListA->Hook; pa; pa = pa->Next)
	s += pa->Cross[p->Cnt];

    if(s < threshold) p->Size = 0; // mark for elimination
 }



 // for each phonem
    // check lines?



 // check each phonem to be referenced by a minimum number of parents
 // similar to previous operation but not redundant
 for(phonemC* p = phonemListA->Hook; p; p = p->Next) if(*p->Sym != '@')
    if(p->Size)
    {
	int s = 0;
	for(phonemC* pa = phonemListA->Hook; pa; pa = pa->Next) s++;

	if(s < 3) p->Size = 0; // mark for elimination
    }



 printf("\n");



 // print starting elements
 printf("\n@");
 for(phonemC* p = phonemListA->Hook; p; p = p->Next) if(p->Size)
    if(*p->Sym == '@')
	printf(" %s", p->Sym+1);




 // print surviving phonems
 for(phonemC* p = phonemListA->Hook; p; p = p->Next) if(p->Size)
 {
    long ref_threshold = 1;
    long cnt_threshold = 2;
    long s = 0;

    for(phonemC* c = phonemListA->Hook; c; c = c->Next)
    	if(p->Cross[c->Cnt] > ref_threshold) s++;
    if(s < cnt_threshold ) continue;


    char* n = p->Sym;
    if(*n == '@') n++;
    printf("\n%s", n);


    // write available children
    for(phonemC* c = phonemListA->Hook; c; c = c->Next)
    	if(p->Cross[c->Cnt] > ref_threshold) printf(" %s", c->Sym);
 }





 printf("\n\n");


 // print phonems
/* for(int n = 2; n <= 5; n++)
 {
    phonemListC* l = phonemListA + n -2;
    for(phonemC* p = l->Hook; p; p = p->Next)
	printf("%6d %10s\n", p->Cnt, p->Sym);
 }
*/

 return 0;
}



//EOF =========================================================================
